﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows;
using System.Windows.Input;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using Color = System.Windows.Media.Color;
using ColorConverter = System.Windows.Media.ColorConverter;

namespace Publishing
{
    /// <summary>
    /// Interaction logic for PrintWindow.xaml
    /// </summary>
    public partial class PrintWindow : Window
    {
        public PrintWindow()
        {
            InitializeComponent();
        }
        public string ShpFilePath { get; set; }

        private void SetupMapWithBlankPage()
        {
            Map1.MapUnit = GeographyUnit.Meter;
            Map1.ZoomLevelSet = new PrinterZoomLevelSet(Map1.MapUnit,

                PrinterHelper.GetPointsPerGeographyUnit(Map1.MapUnit));
            Map1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            Map1.Cursor = Cursors.Cross;
            Map1.Cursor = null;

            var printerOverlay = new PrinterInteractiveOverlay();

            Map1.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);
            Map1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");
            var pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();

            printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);
            Map1.CurrentExtent = AreaBaseShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();
            Map1.CurrentScale = PrinterHelper.GetPointsPerGeographyUnit(Map1.MapUnit);
            Map1.MinimumScale = Map1.ZoomLevelSet.ZoomLevel20.Scale;
        }
        private void AddMapLayer()
        {
            var mapPrinterLayer = new MapPrinterLayer
            {
                MapUnit = GeographyUnit.Meter,
                BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1)),
                PreviewDrawingMode = MapPrinterPreviewDrawingMode.Redraw
            };

            mapPrinterLayer.Open();

            //BingMapsLayer bingLayer = new BingMapsLayer("yourbingkey");
            //bingLayer.MapType = BingMapsMapType.Road;
            //GoogleMapsLayer googleLayer = new GoogleMapsLayer();
            //mapPrinterLayer.Layers.Add(googleLayer);

            var pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 6, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1, PrintingUnit.Inch);
            var rectangleShape1 = AddShpFileToPrintLayer(mapPrinterLayer, @"..\..\ShpFiles\Cardiff_OA_region.shp");


            mapPrinterLayer.MapExtent = rectangleShape1;//new RectangleShape(-10778619, 3868603, -10773962, 3865106);

            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            printerInteractiveOverlay.PrinterLayers.Add("MapLayer", mapPrinterLayer);

        }
        private static RectangleShape AddShpFileToPrintLayer(MapPrinterLayer mapPrinterLayer, string shpFilePath)
        {
            var layer = new ShapeFileFeatureLayer(shpFilePath);

            layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle =
                AreaStyles.CreateSimpleAreaStyle(ToGeoColor(GetColor("#B7008000")));
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            Proj4Projection proj4 = new Proj4Projection()
            {
                InternalProjectionParametersString = Proj4Projection.ConvertPrjToProj4(File.ReadAllText(System.IO.Path.ChangeExtension(shpFilePath, "prj"))),
                ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString()
            };
            proj4.Open();
            layer.FeatureSource.Projection = proj4;

            mapPrinterLayer.Layers.Add(layer);
            layer.Open();
            var rectangleShape1 = layer.GetBoundingBox();
            layer.Close();
            return rectangleShape1;
        }
        private RectangleShape GetPageBoundingBox(PrintingUnit unit)
        {
            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverlay.PrinterLayers["PageLayer"];
            return pagePrinterLayer.GetPosition(unit);
        }
        private void PrintWindow_OnLoaded(object sender, RoutedEventArgs e)
        {
            Map1.Cursor = null;

            try
            {
                //if (string.IsNullOrEmpty(ShpFilePath))
                //    return;

                SetupMapWithBlankPage();
                AddMapLayer();


                Map1.Refresh();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
        }

        private void btnToBitmap_Click(object sender, RoutedEventArgs e)
        {
            PrinterInteractiveOverlay printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = printerInteractiveOverlay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            // Create a bitmap that is the size of the pages bounding box.  The page by default is in points unit
            // system which is very similar to pixels so that ampping is nice.
            Bitmap bitmap = null;
            try
            {
                bitmap = new Bitmap((int)(pagePrinterLayer.GetBoundingBox().Width * 2), (int)(pagePrinterLayer.GetBoundingBox().Height * 2));

                // Create a GdiPlusGeoCanvas and start the drawing
                GdiPlusGeoCanvas gdiPlusGeoCanvas = new GdiPlusGeoCanvas { DrawingQuality = DrawingQuality.HighQuality };
                gdiPlusGeoCanvas.BeginDrawing(bitmap, pagePrinterLayer.GetBoundingBox(), Map1.MapUnit);

                pagePrinterLayer.IsDrawing = true;
                pagePrinterLayer.Draw(gdiPlusGeoCanvas, new Collection<SimpleCandidate>());
                pagePrinterLayer.IsDrawing = false;

                // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
                // except for the PagePrinterLayer                
                Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (PrinterLayer printerLayer in printerInteractiveOverlay.PrinterLayers)
                {
                    printerLayer.IsDrawing = true;
                    if (!(printerLayer is PagePrinterLayer))
                    {
                        printerLayer.Draw(gdiPlusGeoCanvas, labelsInAllLayers);
                    }
                    printerLayer.IsDrawing = false;
                }

                // End the drawing
                gdiPlusGeoCanvas.EndDrawing();

                //Save the resulting bitmap to a file and open the file to show the user
                string filename = "PrintingResults.bmp";
                bitmap.Save(filename);
                Process.Start(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
            finally
            {
                if (bitmap != null) { bitmap.Dispose(); }
            }
        }

        public static Color GetColor(string colorString)
        {
            return (Color)ColorConverter.ConvertFromString(colorString);
        }
        public static GeoColor ToGeoColor(Color color)
        {
            return new GeoColor(
                color.A,
                color.R,
                color.G,
                color.B);
        }
    }
}
